#include <htc.h>
#include <stdio.h>
#include "init.h"
#include "uart.h"

#ifndef SPBRG_VAL
// Unless specified elsewhere, baud rate 4800 for 4MHz clock is assumed
#define SPBRG_VAL 51
#endif

#define UART_RX_BUFFER_SIZE 80

// Stores one line of data as string
unsigned char uart_rxBuffer[UART_RX_BUFFER_SIZE];
// Number of characters in the line - string length
unsigned char uart_rxBufferSize=0;

void uart_setup(void) {
  uart_rxBuffer[0]='\0'; // string null termination

  SPBRG=SPBRG_VAL; // set baud rate
  TXSTA=0x04; // enable high baud rate (BRGH)
  RCSTA=0x90; // enable serial port (SPEN), RX & TX pins and reception (CREN)
}

unsigned char getch() {
  while (true) {
    // Wait for data
    if (RCIF) {
      // Data ready

      // EUSART overrun error occurred
      if (OERR) {
        // Reset the receiver logic
        CREN=0;
        CREN=1;
        continue;
      }

      // Framing error occurred (baud rate disagree)
      if (FERR) {
        // Discard received data that has error
        RCREG;
        continue;
      }

      // Good character
      return RCREG;
    }
  }
}

unsigned char * uart_getLine(void) {
  unsigned char rx=NULL;
  uart_rxBufferSize=0; // reset buffer

  // Fill buffer until line feed
  while (rx!=0x0a) {
    rx=getch(); // get character
    if (uart_rxBufferSize>=UART_RX_BUFFER_SIZE-1) {
      // Buffer overflow - cancel reception
      break;
    }
    uart_rxBuffer[uart_rxBufferSize]=rx; // store character to buffer
    uart_rxBufferSize++;
  }
  uart_rxBuffer[uart_rxBufferSize]='\0'; // string null termination
  return uart_rxBuffer;
}
